﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Threading.Tasks;
using System.ServiceModel.DomainServices.Client;

namespace AsyncQueueSample
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
        }


        /// <summary>
        /// 这是常规的并行方式，多个异步调用彼此是没有关联的，同时发出请求
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btASync_Click(object sender, RoutedEventArgs e)
        {
            var proxy = new localhost.SampleWebServiceSoapClient();
            proxy.HelloWorldCompleted += (o, a) => { MessageBox.Show(a.Result); };
            proxy.HelloWorldAsync();//发出异步请求

            proxy.HelloWorld2Completed += (o, a) => { MessageBox.Show(a.Result); };
            proxy.HelloWorld2Async();


        }
        /// <summary>
        /// 这是常规实现的串行方式，在任务之间手工实现顺序，耦合性很高
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btNormal_Click(object sender, RoutedEventArgs e)
        {
            var proxy = new localhost.SampleWebServiceSoapClient();
            proxy.HelloWorldCompleted += (o, a) =>//第一个任务的回调
            {
                MessageBox.Show(a.Result);

                proxy.HelloWorld2Completed += (o1, a1) =>//第二个任务的回调
                {
                    MessageBox.Show(a1.Result);

                    proxy.HelloWorld3Completed += (o2, a2) =>
                    {
                        MessageBox.Show(a2.Result);
                    };

                    proxy.HelloWorld3Async();

                };

                proxy.HelloWorld2Async();
            };

            proxy.HelloWorldAsync();
        }

        private void btAsyncTaskFactory_Click(object sender, RoutedEventArgs e)
        {
            var proxy = new localhost.SampleWebServiceSoapClient();
            var task1 = new AsyncAction("task 1");
            task1.SetAction(() =>
            {
                proxy.HelloWorldCompleted += (o, a) => { MessageBox.Show(a.Result); task1.OnCompleted(); };
                proxy.HelloWorldAsync();
            });

            var task2 = new AsyncAction("task 2");
            task2.SetAction(() =>
            {
                proxy.HelloWorld2Completed += (o, a) => { MessageBox.Show(a.Result); task2.OnCompleted(); };
                proxy.HelloWorld2Async();
            });


            var task3 = new AsyncAction("task 3");
            task3.SetAction(() =>
            {
                proxy.HelloWorld3Completed += (o, a) => { MessageBox.Show(a.Result); task3.OnCompleted(); };
                proxy.HelloWorld3Async();
            });


            var runner = new AsyncActionRunner(task1, task3, task2);
            runner.Execute();
        }

        async private void btAsyncCTP_Click(object sender, RoutedEventArgs e)
        {
            var proxy = new localhost.SampleWebServiceSoapClient() as localhost.SampleWebServiceSoap;

            var result1 = await Task<localhost.HelloWorldResponse>.Factory.FromAsync(
                proxy.BeginHelloWorld(new localhost.HelloWorldRequest(), null, null),
                proxy.EndHelloWorld);

            MessageBox.Show(result1.Body.HelloWorldResult);

            var result3 = await Task<localhost.HelloWorld3Response>.Factory.FromAsync(
                proxy.BeginHelloWorld3(new localhost.HelloWorld3Request(), null, null),
                proxy.EndHelloWorld3);
            MessageBox.Show(result3.Body.HelloWorld3Result);

            var result2 = await Task<localhost.HelloWorld2Response>.Factory.FromAsync(
                proxy.BeginHelloWorld2(new localhost.HelloWorld2Request(), null, null),
                proxy.EndHelloWorld2);
            MessageBox.Show(result2.Body.HelloWorld2Result);

        }

        private void btRIAService_Click(object sender, RoutedEventArgs e)
        {
            var ctx = new Web.SampleDomainContext();
            ctx.Load<Web.Employee>(ctx.GetEmployeesQuery(), result =>
            {
                MessageBox.Show(result.Entities.FirstOrDefault().FirstName);
            }, true);

            MessageBox.Show("加载完成");
        }

        async private void btAsyncCTPRIA_Click(object sender, RoutedEventArgs e)
        {
            var ctx = new Web.SampleDomainContext();
            var result = await ctx.Load<Web.Employee>(ctx.GetEmployeesQuery()).AsTask();
            MessageBox.Show(result.Entities.FirstOrDefault().FirstName);



            MessageBox.Show("加载完成");
        }

    }


    public static class OperationExtensions
    {
        public static Task<T> AsTask<T>(this T operation)
          where T : OperationBase
        {
            TaskCompletionSource<T> tcs =
              new TaskCompletionSource<T>(operation.UserState);

            operation.Completed += (sender, e) =>
            {
                if (operation.HasError && !operation.IsErrorHandled)
                {
                    tcs.TrySetException(operation.Error);
                    operation.MarkErrorAsHandled();
                }
                else if (operation.IsCanceled)
                {
                    tcs.TrySetCanceled();
                }
                else
                {
                    tcs.TrySetResult(operation);
                }
            };

            return tcs.Task;
        }
    }

}
